import { createApiRef, createRouteRef, createPlugin, createApiFactory, createRoutableExtension, createComponentExtension, useApi, googleAuthApiRef, errorApiRef } from '@backstage/core-plugin-api';
import fetch from 'cross-fetch';
import React, { useEffect } from 'react';
import { Routes, Route } from 'react-router-dom';
import { Table, InfoCard, StructuredMetadataTable, ContentHeader, SupportButton, MissingAnnotationEmptyState } from '@backstage/core-components';
import { FormControl, InputLabel, Select, Input, MenuItem, Checkbox, ListItemText, Grid, Box, Typography, Table as Table$1, TableBody, TableRow, TableCell, Link, LinearProgress, makeStyles, TableContainer, Card } from '@material-ui/core';
import { useAsyncRetry, useLocalStorage } from 'react-use';
import moment from 'moment';
import { useEntity } from '@backstage/plugin-catalog-react';
import ExternalLinkIcon from '@material-ui/icons/Launch';

const firebaseFunctionsApiRef = createApiRef({
  id: "plugin.firebasefunctions.service",
  description: "Used by the firebase functions plugin to make requests"
});

class FetchError extends Error {
  get name() {
    return this.constructor.name;
  }
  static async forResponse(resp) {
    return new FetchError(`Request failed with status code ${resp.status}.
Reason: ${await resp.text()}`);
  }
}
async function fetchWrapper(url, init) {
  const resp = await fetch(`${url}`, init);
  if (!resp.ok)
    throw await FetchError.forResponse(resp);
  return await resp.json();
}
class FirebaseFunctionsClient {
  async getFunction({googleIdToken, functionSlug}) {
    const url = `https://cloudfunctions.googleapis.com/v1/${functionSlug}`;
    const init = {
      method: "get",
      headers: {
        Authorization: `Bearer ${googleIdToken}`
      }
    };
    const resp = await fetchWrapper(url, init);
    return {
      name: resp.name.split("/").pop(),
      urlTrigger: resp.httpsTrigger.url,
      status: resp.status,
      updateTime: resp.updateTime,
      runtime: resp.runtime,
      availableMemoryMb: resp.availableMemoryMb,
      project: functionSlug.split("/").slice(-5)[0],
      region: resp.name.split("/").slice(-3)[0],
      labels: resp.labels,
      envVariables: resp.environmentVariables,
      fullName: resp.name
    };
  }
  async listFunctions({googleIdToken, projects}) {
    let functionData = [];
    for (let i = 0; i < projects.length; i++) {
      const project = projects[i];
      let url = `https://cloudfunctions.googleapis.com/v1/projects/${project}/locations/-/functions?pageSize=50`;
      const init = {
        method: "get",
        headers: {
          Authorization: `Bearer ${googleIdToken}`
        }
      };
      const fetchedData = [];
      let resp = null;
      do {
        if (resp) {
          url = `${url}?pageToken=$${encodeURIComponent(resp.nextPageToken)}`;
        }
        resp = await fetchWrapper(url, init);
        fetchedData.push(...resp.functions);
      } while (resp && resp.nextPageToken);
      functionData = functionData.concat(fetchedData.map((r) => ({
        name: r.name.split("/").pop(),
        urlTrigger: r.httpsTrigger.url,
        status: r.status,
        updateTime: r.updateTime,
        runtime: r.runtime,
        availableMemoryMb: r.availableMemoryMb,
        project,
        region: r.name.split("/").slice(-3)[0],
        labels: r.labels,
        envVariables: r.environmentVariables,
        fullName: r.name
      })));
    }
    return {functionData};
  }
}

const entityContentRouteRef = createRouteRef({
  title: "Firebase functions Entity Content"
});
const firebaseFunctionsPlugin = createPlugin({
  id: "firebase-functions",
  apis: [
    createApiFactory(firebaseFunctionsApiRef, new FirebaseFunctionsClient())
  ],
  routes: {
    entityContent: entityContentRouteRef
  }
});
const EntityFirebaseFunctionsContent = firebaseFunctionsPlugin.provide(createRoutableExtension({
  component: () => Promise.resolve().then(function () { return Router$1; }).then((m) => m.Router),
  mountPoint: entityContentRouteRef
}));
const EntityFirebaseFunctionsCard = firebaseFunctionsPlugin.provide(createComponentExtension({
  component: {
    lazy: () => Promise.resolve().then(function () { return FirebaseFunctionWidget$1; }).then((m) => m.FirebaseFunctionWidget)
  }
}));

function useFirebaseFunctions(projects) {
  const googleAuth = useApi(googleAuthApiRef);
  const firebaseFunctionsApi = useApi(firebaseFunctionsApiRef);
  const errorApi = useApi(errorApiRef);
  const {loading, value: functionsData, error, retry} = useAsyncRetry(async () => {
    const googleIdToken = await googleAuth.getAccessToken([
      "https://www.googleapis.com/auth/cloud-platform"
    ]);
    try {
      const firebaseFunctions = await firebaseFunctionsApi.listFunctions({
        googleIdToken,
        projects
      });
      return firebaseFunctions.functionData;
    } catch (err) {
      errorApi.post(err);
      throw new Error(err);
    }
  }, [projects]);
  return {
    loading,
    functionsData,
    error,
    retry
  };
}

const StateContext = React.createContext([]);
const STORAGE_KEY = `${firebaseFunctionsApiRef.id}.settings`;
const initialState = {
  projects: [],
  entity: null
};
const ContextProvider = ({entity, children}) => {
  const [settings, setSettings] = useLocalStorage(STORAGE_KEY, initialState);
  if (settings === void 0) {
    throw new Error("Firebase functions plugin settings is undefined");
  }
  useEffect(() => {
    if (settings.entity !== entity) {
      setSettings({...settings, entity});
    }
  }, [entity]);
  return /* @__PURE__ */ React.createElement(StateContext.Provider, {
    value: [settings, setSettings]
  }, /* @__PURE__ */ React.createElement(React.Fragment, null, children));
};

const useSettings = () => {
  const context = React.useContext(StateContext);
  if (context === void 0) {
    throw new Error("You must use 'useSettings' hook inside ContextProvider");
  }
  return context;
};

const FIREBASE_FUNCTION_IDS = "cloud.google.com/function-ids";
const useFunctionIds = () => {
  var _a, _b;
  const {entity} = useEntity();
  const rawProjects = (_b = (_a = entity == null ? void 0 : entity.metadata.annotations) == null ? void 0 : _a[FIREBASE_FUNCTION_IDS]) != null ? _b : "";
  const functions = rawProjects.split(",").map((p) => p.trim());
  const availableProjects = [...new Set(functions.map((f) => f.split("/")[1]))].filter((it) => it);
  return {availableProjects, functions};
};

const getElapsedTime = (start) => {
  return moment(start).fromNow();
};
const columnDefinitions = [
  {
    title: "Name",
    field: "name",
    render: (row) => {
      return /* @__PURE__ */ React.createElement(Box, {
        fontWeight: "fontWeightBold"
      }, /* @__PURE__ */ React.createElement(Link, {
        target: "_blank",
        href: `https://console.cloud.google.com/functions/details/${row.region}/${row.name}?project=${row.project}`
      }, row.name));
    }
  },
  {
    title: "Project",
    field: "project",
    render: (row) => /* @__PURE__ */ React.createElement(Typography, {
      variant: "body2",
      noWrap: true
    }, row.project)
  },
  {
    title: "Status",
    field: "status",
    width: "150px",
    render: (row) => /* @__PURE__ */ React.createElement(Typography, {
      variant: "body2",
      noWrap: true
    }, row.status)
  },
  {
    title: "Region",
    field: "region",
    width: "150px",
    render: (row) => /* @__PURE__ */ React.createElement(Typography, {
      variant: "body2",
      noWrap: true
    }, row.region)
  },
  {
    title: "Last modified",
    field: "updateTime",
    width: "150px",
    render: (row) => /* @__PURE__ */ React.createElement(Typography, {
      variant: "body2",
      noWrap: true
    }, getElapsedTime(row.updateTime))
  },
  {
    title: "Runtime",
    field: "runtime",
    width: "150px",
    render: (row) => /* @__PURE__ */ React.createElement(Typography, {
      variant: "body2",
      noWrap: true
    }, row.runtime)
  },
  {
    title: "Memory",
    field: "availableMemoryMb",
    width: "150px",
    render: (row) => /* @__PURE__ */ React.createElement(Typography, {
      variant: "body2",
      noWrap: true
    }, row.availableMemoryMb, " MB")
  },
  {
    title: "Logs",
    field: "",
    width: "150px",
    render: (row) => {
      return /* @__PURE__ */ React.createElement(Link, {
        href: `https://console.cloud.google.com/logs/viewer?project=${row.project}&resource=cloud_function%2Ffunction_name%2F${row.name}%2Fregion%2F${row.region}`,
        target: "_blank"
      }, "view logs");
    }
  }
];
const FirebaseFunctionsPageTable = () => {
  var _a;
  const [settings, setSettings] = useSettings();
  const {
    availableProjects,
    functions: whitelistedFunctions
  } = useFunctionIds();
  const firebaseFunctions = useFirebaseFunctions(settings.projects);
  const functionsData = (_a = firebaseFunctions.functionsData) == null ? void 0 : _a.filter(filterFireabseFuntions(whitelistedFunctions));
  useEffect(() => {
    firebaseFunctions.retry();
  }, [settings.projects]);
  const handleChange = (event) => {
    setSettings({...settings, projects: event.target.value});
  };
  const projectSelect = /* @__PURE__ */ React.createElement(FormControl, null, /* @__PURE__ */ React.createElement(InputLabel, {
    id: "project-ids-label"
  }, "Select projects"), /* @__PURE__ */ React.createElement(Select, {
    style: {minWidth: "150px"},
    labelId: "project-ids-label",
    id: "project-ids",
    multiple: true,
    renderValue: (selected) => selected.join(", "),
    value: settings.projects,
    onChange: handleChange,
    input: /* @__PURE__ */ React.createElement(Input, null)
  }, availableProjects == null ? void 0 : availableProjects.map((name) => /* @__PURE__ */ React.createElement(MenuItem, {
    key: name,
    value: name
  }, /* @__PURE__ */ React.createElement(Checkbox, {
    checked: settings.projects.indexOf(name) > -1
  }), /* @__PURE__ */ React.createElement(ListItemText, {
    primary: name
  })))));
  return /* @__PURE__ */ React.createElement(Grid, {
    container: true,
    spacing: 1,
    direction: "column"
  }, /* @__PURE__ */ React.createElement(Grid, {
    item: true
  }, projectSelect), /* @__PURE__ */ React.createElement(Grid, {
    item: true
  }, /* @__PURE__ */ React.createElement(Table, {
    isLoading: firebaseFunctions.loading || firebaseFunctions.loading,
    options: {
      padding: "dense"
    },
    data: functionsData != null ? functionsData : [],
    columns: columnDefinitions,
    localization: getLocalizationObject(settings, firebaseFunctions),
    detailPanel: DetailPanel
  })));
};
function getLocalizationObject(settings, tableProps) {
  let message = void 0;
  switch (true) {
    case settings.projects.length === 0:
      message = "Select projects to fetch data";
      break;
    case tableProps.loading:
      message = "loading";
      break;
    case !!tableProps.error:
      message = `error occured while loading data: ${tableProps.error}`;
      break;
  }
  return message ? {
    body: {
      emptyDataSourceMessage: message
    }
  } : void 0;
}
function DetailPanel(rowData) {
  return /* @__PURE__ */ React.createElement(Box, {
    display: "flex",
    p: 1
  }, /* @__PURE__ */ React.createElement(Box, {
    p: 1,
    maxWidth: "50%"
  }, /* @__PURE__ */ React.createElement(Typography, null, "Env variables:"), /* @__PURE__ */ React.createElement(Table$1, {
    size: "small",
    "aria-label": "env-variables"
  }, /* @__PURE__ */ React.createElement(TableBody, null, rowData.envVariables ? Object.entries(rowData.envVariables).map((entry) => /* @__PURE__ */ React.createElement(TableRow, {
    key: entry[0]
  }, /* @__PURE__ */ React.createElement(TableCell, null, entry[0]), /* @__PURE__ */ React.createElement(TableCell, null, entry[1]))) : "no env variables found"))), /* @__PURE__ */ React.createElement(Box, {
    p: 1,
    maxWidth: "50%"
  }, /* @__PURE__ */ React.createElement(Typography, null, "labels:"), /* @__PURE__ */ React.createElement(Table$1, {
    size: "small",
    "aria-label": "labels"
  }, /* @__PURE__ */ React.createElement(TableBody, null, rowData.envVariables ? Object.entries(rowData.labels).map((entry) => /* @__PURE__ */ React.createElement(TableRow, {
    key: entry[0]
  }, /* @__PURE__ */ React.createElement(TableCell, null, entry[0]), /* @__PURE__ */ React.createElement(TableCell, null, entry[1]))) : "no labels found"))));
}
function filterFireabseFuntions(whitelistedFunctions) {
  return (element) => {
    return whitelistedFunctions.includes(element.fullName);
  };
}

const isOnlyOneFirebaseFunction = (entity) => {
  var _a, _b;
  const rawProjects = (_b = (_a = entity == null ? void 0 : entity.metadata.annotations) == null ? void 0 : _a[FIREBASE_FUNCTION_IDS]) != null ? _b : void 0;
  if (!rawProjects) {
    return false;
  }
  const functions = rawProjects.split(",").map((p) => p.trim());
  return functions.length === 1;
};
const isMoreThanOneFirebaseFunction = (entity) => {
  var _a, _b;
  const rawProjects = (_b = (_a = entity == null ? void 0 : entity.metadata.annotations) == null ? void 0 : _a[FIREBASE_FUNCTION_IDS]) != null ? _b : "";
  const functions = rawProjects.split(",").map((p) => p.trim());
  return functions.length > 1;
};

function useSingleFirebaseFunction(functionSlug) {
  const googleAuth = useApi(googleAuthApiRef);
  const firebaseFunctionsApi = useApi(firebaseFunctionsApiRef);
  const errorApi = useApi(errorApiRef);
  const {loading, value: functionData, error, retry} = useAsyncRetry(async () => {
    const googleIdToken = await googleAuth.getAccessToken([
      "https://www.googleapis.com/auth/cloud-platform"
    ]);
    try {
      const ResponseData = await firebaseFunctionsApi.getFunction({
        googleIdToken,
        functionSlug
      });
      return ResponseData;
    } catch (err) {
      errorApi.post(err);
      throw new Error(err);
    }
  });
  return {
    loading,
    functionData,
    error,
    retry
  };
}

const FirebaseFunctionDetailsPage = () => {
  const {functions: whitelistedFunctions} = useFunctionIds();
  const {loading, functionData, error} = useSingleFirebaseFunction(whitelistedFunctions[0]);
  if (error) {
    return /* @__PURE__ */ React.createElement(Typography, null, error.message);
  }
  return /* @__PURE__ */ React.createElement(React.Fragment, null, loading || !functionData ? /* @__PURE__ */ React.createElement(LinearProgress, null) : /* @__PURE__ */ React.createElement(Grid, {
    container: true,
    spacing: 1
  }, /* @__PURE__ */ React.createElement(Grid, {
    item: true,
    md: 6
  }, /* @__PURE__ */ React.createElement(InfoCard, {
    title: /* @__PURE__ */ React.createElement(Link, {
      href: `https://console.cloud.google.com/functions/details/${functionData.region}/${functionData.name}?project=${functionData.project}`,
      target: "_blank"
    }, functionData.name)
  }, /* @__PURE__ */ React.createElement(StructuredMetadataTable, {
    metadata: {
      status: functionData.status,
      "last modified": moment(functionData.updateTime).fromNow(),
      project: functionData.project,
      region: functionData.region,
      runtime: functionData.runtime,
      memory: `${functionData.availableMemoryMb} MB`,
      logs: /* @__PURE__ */ React.createElement(Link, {
        href: `https://console.cloud.google.com/logs/viewer?project=${functionData.project}&resource=cloud_function%2Ffunction_name%2F${functionData.name}%2Fregion%2F${functionData.region}`,
        target: "_blank"
      }, "view logs")
    }
  }))), /* @__PURE__ */ React.createElement(Grid, {
    item: true,
    md: 6
  }, /* @__PURE__ */ React.createElement(InfoCard, {
    title: "Labels"
  }, /* @__PURE__ */ React.createElement(StructuredMetadataTable, {
    metadata: functionData.labels
  })))));
};

const FirebaseFunctionsPage = ({entity}) => {
  return /* @__PURE__ */ React.createElement(ContextProvider, {
    entity
  }, /* @__PURE__ */ React.createElement(ContentHeader, {
    title: "Firebase Functions"
  }, /* @__PURE__ */ React.createElement(SupportButton, null, "Plugin to show a project's firebase functions")), isMoreThanOneFirebaseFunction(entity) ? /* @__PURE__ */ React.createElement(FirebaseFunctionsPageTable, null) : /* @__PURE__ */ React.createElement(FirebaseFunctionDetailsPage, null));
};

const isFirebaseFunctionsAvailable = (entity) => {
  var _a;
  return Boolean((_a = entity == null ? void 0 : entity.metadata.annotations) == null ? void 0 : _a[FIREBASE_FUNCTION_IDS]);
};
const Router = (_props) => {
  const {entity} = useEntity();
  return !isFirebaseFunctionsAvailable(entity) ? /* @__PURE__ */ React.createElement(MissingAnnotationEmptyState, {
    annotation: FIREBASE_FUNCTION_IDS
  }) : /* @__PURE__ */ React.createElement(Routes, null, /* @__PURE__ */ React.createElement(Route, {
    path: `/${entityContentRouteRef.path}`,
    element: /* @__PURE__ */ React.createElement(FirebaseFunctionsPage, {
      entity
    })
  }));
};

var Router$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  isFirebaseFunctionsAvailable: isFirebaseFunctionsAvailable,
  Router: Router
});

const useStyles = makeStyles({
  externalLinkIcon: {
    fontSize: "inherit"
  },
  link: {
    textAlign: "center"
  }
});
const FirebaseFunctionDetailsCard = ({
  firebaseFunction
}) => {
  const classes = useStyles();
  return /* @__PURE__ */ React.createElement(StructuredMetadataTable, {
    metadata: {
      name: /* @__PURE__ */ React.createElement(Link, {
        href: `https://console.cloud.google.com/functions/details/${firebaseFunction.region}/${firebaseFunction.name}?project=${firebaseFunction.project}`,
        target: "_blank",
        className: classes.link
      }, firebaseFunction.name, /* @__PURE__ */ React.createElement(ExternalLinkIcon, {
        className: classes.externalLinkIcon
      })),
      status: firebaseFunction.status,
      "last modified": moment(firebaseFunction.updateTime).fromNow(),
      project: firebaseFunction.project,
      "link to logs": /* @__PURE__ */ React.createElement(Link, {
        href: `https://console.cloud.google.com/logs/viewer?project=${firebaseFunction.project}&resource=cloud_function%2Ffunction_name%2F${firebaseFunction.name}%2Fregion%2F${firebaseFunction.region}`,
        target: "_blank"
      }, "view logs")
    }
  });
};

const FirebaseFunctionWidgetPage = () => {
  const {functions: whitelistedFunctions} = useFunctionIds();
  const {loading, functionData, error} = useSingleFirebaseFunction(whitelistedFunctions[0]);
  if (error) {
    return /* @__PURE__ */ React.createElement(TableContainer, {
      component: Card
    }, /* @__PURE__ */ React.createElement(InfoCard, {
      title: "Firebase Function Details"
    }, /* @__PURE__ */ React.createElement(Typography, null, error.message), ";"));
  }
  return /* @__PURE__ */ React.createElement(TableContainer, {
    component: Card
  }, /* @__PURE__ */ React.createElement(InfoCard, {
    title: "Firebase Function Details"
  }, loading || !functionData ? /* @__PURE__ */ React.createElement(LinearProgress, null) : /* @__PURE__ */ React.createElement(FirebaseFunctionDetailsCard, {
    firebaseFunction: functionData
  })));
};
const FirebaseFunctionWidget = (_props) => {
  const {entity} = useEntity();
  return /* @__PURE__ */ React.createElement(ContextProvider, {
    entity
  }, /* @__PURE__ */ React.createElement(FirebaseFunctionWidgetPage, null));
};

var FirebaseFunctionWidget$1 = /*#__PURE__*/Object.freeze({
  __proto__: null,
  FirebaseFunctionWidget: FirebaseFunctionWidget
});

export { EntityFirebaseFunctionsCard, EntityFirebaseFunctionsContent, FirebaseFunctionWidget, FirebaseFunctionsClient, Router, entityContentRouteRef, firebaseFunctionsApiRef, firebaseFunctionsPlugin, isFirebaseFunctionsAvailable, isMoreThanOneFirebaseFunction, isOnlyOneFirebaseFunction, isFirebaseFunctionsAvailable as isPluginApplicableToEntity };
//# sourceMappingURL=index.esm.js.map
